import datetime
import unittest

from submerger.subtitle import Subtitle


class TestSubtitleParsing(unittest.TestCase):
    def test_from_srt(self):
        expected = []
        expected.append(Subtitle(datetime.timedelta(
            seconds=1), datetime.timedelta(seconds=3),
            '''some
text
here
for
testing'''))

        actual = Subtitle.from_srt(f'1\n{str(expected[0])}')

        self.assertEqual(str(actual[0]), str(expected[0]))

    def test_to_srt(self):
        expected = '''1
00:01:00,000 --> 00:02:00,000
111

2
00:02:01,000 --> 00:03:00,000
222

3
00:03:00,000 --> 00:04:00,000
333

4
00:04:00,000 --> 00:05:00,000
444

5
00:05:00,000 --> 00:06:00,000
555

6
00:06:00,000 --> 00:07:00,000
666

7
00:07:00,000 --> 00:08:00,000
777

8
00:08:00,000 --> 00:09:00,000
888

9
01:00:00,000 --> 01:01:00,000
999

'''
        subtitle_list = []
        subtitle_list.extend(Subtitle.from_srt(expected))

        actual = Subtitle.to_srt(subtitle_list)

        self.assertEqual(actual, expected)
        self.assertListEqual(Subtitle.from_srt(actual),
                             Subtitle.from_srt(expected))

    def test___str__(self):
        actual1 = Subtitle(datetime.timedelta(
            seconds=123, milliseconds=456), datetime.timedelta(seconds=321, milliseconds=54),
            'some text')

        self.assertEqual(
            str(actual1), '00:02:03,456 --> 00:05:21,054\nsome text\n\n')

        actual2 = Subtitle(datetime.timedelta(
            seconds=1234, milliseconds=2), datetime.timedelta(seconds=4321),
            'some text')

        self.assertEqual(
            str(actual2), '00:20:34,002 --> 01:12:01,000\nsome text\n\n')



import unittest

from submerger.test.execute_command import execute_command


class TestTyping(unittest.IsolatedAsyncioTestCase):
    async def test_with_mypy(self):
        return_code, stdout, _ = await execute_command(
            'mypy',
            'submerger',
            'scripts',
            'bin/submerge',
        )
        self.assertEqual(return_code, 0, f'mypy exited with errors\n{stdout}')


class TestCode(unittest.IsolatedAsyncioTestCase):
    async def test_with_pylint(self):
        return_code, stdout, _ = await execute_command(
            'pylint',
            'submerger',
            'scripts',
            'bin/submerge',
        )
        self.assertEqual(
            return_code, 0, f'pylint exited with errors\n{stdout}')



import asyncio
from pathlib import Path
from typing import Tuple


async def execute_command(
        command: str | Path,
        *args: str | Path,
) -> Tuple[int | None, str, str]:
    proc = await asyncio.create_subprocess_exec(
        command,
        *args,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE)

    stdout, stderr = await proc.communicate()
    return proc.returncode, stdout.decode(), stderr.decode()



from typing import List

from submerger.subtitle import Subtitle
from submerger.subtitle_format import SubtitleFormat


def merge_subtitles(
        base_srt: str,
        base_srt_tags: SubtitleFormat,
        secondary_srt: str,
        secondary_srt_tags: SubtitleFormat,
) -> str:
    subtitle_list: List[Subtitle] = []

    for subtitle in Subtitle.from_srt(base_srt):
        subtitle_list.append(Subtitle(
            subtitle.start_time,
            subtitle.end_time,
            base_srt_tags.format(subtitle.text),
        ))

    for subtitle in Subtitle.from_srt(secondary_srt):
        subtitle_list.append(Subtitle(
            subtitle.start_time,
            subtitle.end_time,
            secondary_srt_tags.format(subtitle.text),
        ))

    return Subtitle.to_srt(subtitle_list)



import unittest

from submerger.merge_subtitles import merge_subtitles
from submerger.subtitle_format import SubtitleFormat


class TestSubtitleMerging(unittest.TestCase):
    def test_merge_subtitles(self):
        self.assertEqual(merge_subtitles(
            '''1
00:00:08,833 --> 00:00:14,916
Black holes are considered to be
the hellmouths of the universe.

2
00:00:15,625 --> 00:00:19,083
Those who fall inside disappear.

3
00:00:19,791 --> 00:00:20,625
Forever.

4
00:00:25,000 --> 00:00:26,625
But whereto?

''',
            SubtitleFormat().set_position(SubtitleFormat.Position.BOTTOM_CENTER),
            '''1
00:00:08,833 --> 00:00:14,916
[Wissenschaftler]  Schwarze Löcher gelten
als die Höllenschlunde des Universums.

2
00:00:15,625 --> 00:00:19,708
Wer hineinfällt, verschwindet.

3
00:00:19,791 --> 00:00:21,458
Für immer.

4
00:00:25,000 --> 00:00:26,625
Aber wohin?

''',
            SubtitleFormat().set_position(
                SubtitleFormat.Position.TOP_CENTER).set_color('#333').set_italic(),
        ),
            r'''1
00:00:08,833 --> 00:00:14,916
{\an2}Black holes are considered to be
the hellmouths of the universe.

2
00:00:08,833 --> 00:00:14,916
<font color=#333><i>{\an8}[Wissenschaftler]  Schwarze Löcher gelten
als die Höllenschlunde des Universums.</i></font>

3
00:00:15,625 --> 00:00:19,083
{\an2}Those who fall inside disappear.

4
00:00:15,625 --> 00:00:19,708
<font color=#333><i>{\an8}Wer hineinfällt, verschwindet.</i></font>

5
00:00:19,791 --> 00:00:20,625
{\an2}Forever.

6
00:00:19,791 --> 00:00:21,458
<font color=#333><i>{\an8}Für immer.</i></font>

7
00:00:25,000 --> 00:00:26,625
{\an2}But whereto?

8
00:00:25,000 --> 00:00:26,625
<font color=#333><i>{\an8}Aber wohin?</i></font>

'''
        )



from enum import Enum


class SubtitleFormat:
    class Position(Enum):
        BOTTOM_LEFT = '1'
        BOTTOM_CENTER = '2'
        BOTTOM_RIGHT = '3'
        MIDDLE_LEFT = '4'
        MIDDLE_CENTER = '5'
        MIDDLE_RIGHT = '6'
        TOP_LEFT = '7'
        TOP_CENTER = '8'
        TOP_RIGHT = '9'

    def __init__(self) -> None:
        self.position: SubtitleFormat.Position | None = None
        self.color: str | None = None
        self.italic: bool = False

    def set_position(self, position: Position) -> 'SubtitleFormat':
        self.position = position
        return self

    def set_color(self, color: str) -> 'SubtitleFormat':
        self.color = color
        return self

    def set_italic(self) -> 'SubtitleFormat':
        self.italic = True
        return self

    def format(self, text: str) -> str:
        if self.position is not None:
            text = rf'{{\an{self.position.value}}}{text}'

        if self.italic:
            text = f'<i>{text}</i>'

        if self.color is not None:
            text = f'<font color={self.color}>{text}</font>'

        return text



import unittest

from submerger.subtitle_format import SubtitleFormat


TEXT = '''
SOME TEXT  
Якийсь текст 
óõõ
¥ · £ · € · $ · ¢ · ₡ · ₢ · ₣ · ₤ · ₥ · ₦ · ₧ · ₨ · ₩ · ₪ · ₫ · ₭ · ₮ · ₯ · ₹  
ᚠᛇᚻ᛫ᛒᛦᚦ᛫ᚠᚱᚩᚠᚢᚱ᛫ᚠᛁᚱᚪ᛫ᚷᛖᚻᚹᛦᛚᚳᚢᛗ
ඞ
'''


class TestSubtitleFormat(unittest.TestCase):
    def test_text_does_not_change_without_specifying_format(self):
        self.assertEqual(
            SubtitleFormat().format(TEXT),
            TEXT,
        )

    def test_format(self):
        self.assertEqual(
            SubtitleFormat().set_italic().set_position(
                SubtitleFormat.Position.TOP_CENTER).set_color('#333').format(TEXT),
            fr'<font color=#333><i>{{\an8}}{TEXT}</i></font>',
        )
        self.assertEqual(
            SubtitleFormat().set_italic().set_position(
                SubtitleFormat.Position.TOP_RIGHT).set_color('RED').format(TEXT),
            fr'<font color=RED><i>{{\an9}}{TEXT}</i></font>',
        )

    def test_format_idempotence(self):
        self.assertEqual(
            SubtitleFormat().set_position(SubtitleFormat.Position.TOP_CENTER).set_color(
                '#696969').set_position(SubtitleFormat.Position.BOTTOM_RIGHT).set_color(
                    '#333').set_italic().set_italic().set_italic().format(TEXT),
            fr'<font color=#333><i>{{\an3}}{TEXT}</i></font>',
        )

    def test_format_position(self):
        self.assertEqual(
            SubtitleFormat().set_position(SubtitleFormat.Position.BOTTOM_LEFT).format(TEXT),
            fr'{{\an1}}{TEXT}',
        )
        self.assertEqual(
            SubtitleFormat().set_position(SubtitleFormat.Position.BOTTOM_CENTER).format(TEXT),
            fr'{{\an2}}{TEXT}',
        )
        self.assertEqual(
            SubtitleFormat().set_position(SubtitleFormat.Position.BOTTOM_RIGHT).format(TEXT),
            fr'{{\an3}}{TEXT}',
        )
        self.assertEqual(
            SubtitleFormat().set_position(SubtitleFormat.Position.MIDDLE_LEFT).format(TEXT),
            fr'{{\an4}}{TEXT}',
        )
        self.assertEqual(
            SubtitleFormat().set_position(SubtitleFormat.Position.MIDDLE_CENTER).format(TEXT),
            fr'{{\an5}}{TEXT}',
        )
        self.assertEqual(
            SubtitleFormat().set_position(SubtitleFormat.Position.MIDDLE_RIGHT).format(TEXT),
            fr'{{\an6}}{TEXT}',
        )
        self.assertEqual(
            SubtitleFormat().set_position(SubtitleFormat.Position.TOP_LEFT).format(TEXT),
            fr'{{\an7}}{TEXT}',
        )
        self.assertEqual(
            SubtitleFormat().set_position(SubtitleFormat.Position.TOP_CENTER).format(TEXT),
            fr'{{\an8}}{TEXT}',
        )
        self.assertEqual(
            SubtitleFormat().set_position(SubtitleFormat.Position.TOP_RIGHT).format(TEXT),
            fr'{{\an9}}{TEXT}',
        )



from setuptools import setup, find_packages

setup(
    name='submerger',
    version='1.0.0',
    description='',

    author='Andrii Valiukh',
    author_email='andr.you.ay@gmail.com',

    packages=find_packages(),
    python_requires=">=3.10",
    install_requires=[],
    extras_require={
        'dev': [
            'mypy',
            'pylint',
        ]
    },
    scripts=['bin/submerge'],
)



from datetime import timedelta
from typing import List
import re


class Subtitle:
    def __init__(self, start_time: timedelta, end_time: timedelta, text: str) -> None:
        self.__start_time = start_time
        self.__end_time = end_time
        self.__text = text

    @property
    def start_time(self) -> timedelta:
        return self.__start_time

    @property
    def end_time(self) -> timedelta:
        return self.__end_time

    @property
    def text(self) -> str:
        return self.__text

    def __eq__(self, other: object) -> bool:

        if not isinstance(other, Subtitle):
            return NotImplemented

        # pylint: disable=protected-access
        return (self.__start_time == other.__start_time and
                self.__end_time == other.__end_time and
                self.__text == other.__text)
        # pylint: enable=protected-access

    def __str__(self) -> str:
        '''
        Returns subtitle's time line and text:
        '00:01:00,000 --> 00:02:00,000'
        SOME TEXT
        '''
        start_hours = ((self.__start_time.seconds // 3600) +
                       (self.__start_time.days * 24))
        start_minutes = (self.__start_time.seconds % 3600) // 60
        start_seconds = self.__start_time.seconds % 60
        start_milliseconds = self.__start_time.microseconds // 1000

        end_hours = ((self.__end_time.seconds // 3600) +
                     (self.__end_time.days * 24))
        end_minutes = (self.__end_time.seconds % 3600) // 60
        end_seconds = self.__end_time.seconds % 60
        end_milliseconds = self.__end_time.microseconds // 1000

        # pylint: disable-next=line-too-long
        return f'''{start_hours:02d}:{start_minutes:02d}:{start_seconds:02d},{start_milliseconds:03d} --> {end_hours:02d}:{end_minutes:02d}:{end_seconds:02d},{end_milliseconds:03d}
{self.__text}

'''

    @classmethod
    def from_srt(cls, raw_text: str) -> List['Subtitle']:
        pattern = re.compile(
            # pylint: disable-next=line-too-long
            r'\d+\n(?P<start_hours>\d{2}):(?P<start_minutes>\d{2}):(?P<start_seconds>\d{2}),(?P<start_milliseconds>\d{3}) --> (?P<end_hours>\d{2}):(?P<end_minutes>\d{2}):(?P<end_seconds>\d{2}),(?P<end_milliseconds>\d{3})\n(?P<text>.+(\n.+)*?)\n{2}')

        subtitle_list: List[Subtitle] = []
        for match in pattern.finditer(raw_text):
            start_time = timedelta(
                hours=int(match.group('start_hours')),
                minutes=int(match.group('start_minutes')),
                seconds=int(match.group('start_seconds')),
                milliseconds=int(match.group('start_milliseconds')),
            )

            end_time = timedelta(
                hours=int(match.group('end_hours')),
                minutes=int(match.group('end_minutes')),
                seconds=int(match.group('end_seconds')),
                milliseconds=int(match.group('end_milliseconds')),
            )

            text = match.group('text')

            subtitle_list.append(Subtitle(start_time, end_time, text))
        return subtitle_list

    @staticmethod
    def to_srt(subtitle_list: List['Subtitle']) -> str:
        subtitle_list = sorted(
            subtitle_list, key=lambda Subtitle: Subtitle.__start_time)

        return ''.join(f'{subtitle_counter}\n{sub}'
                       for subtitle_counter, sub in enumerate(subtitle_list, 1))



import shutil
import unittest
from pathlib import Path

from submerger.test.execute_command import execute_command

TEST_DATA_PATH = Path('submerger/test/data')
TEST_OUTPUT_PATH = Path('test_outputs')
DUALSUB_EXPECTED_PATH = TEST_DATA_PATH / 'dualsub_expected.srt'
TEST_GLOBAL_MERGING_PATH = TEST_DATA_PATH / 'global_merging'


class TestIntegration(unittest.IsolatedAsyncioTestCase):
    def setUp(self) -> None:
        TEST_OUTPUT_PATH.mkdir()

    def tearDown(self) -> None:
        shutil.rmtree(TEST_OUTPUT_PATH)
        Path.unlink(TEST_GLOBAL_MERGING_PATH / 'ep1.srt', missing_ok=True)
        Path.unlink(TEST_GLOBAL_MERGING_PATH / 'ep2.srt', missing_ok=True)

    async def test_submerger_from_srt(self) -> None:
        dualsub_from_srt_path = TEST_OUTPUT_PATH / 'dualsub_from_srt.srt'

        return_code, _, _ = await execute_command(
            'submerge',
            '--subtitles', TEST_DATA_PATH / 'eng.srt', TEST_DATA_PATH / 'ger.srt',
            '--output', dualsub_from_srt_path,
        )
        self.assertEqual(return_code, 0)

        with (open(dualsub_from_srt_path, encoding='utf-8')
                as dualsub_from_srt,
                open(DUALSUB_EXPECTED_PATH, encoding='utf-8')
                as dualsub_expected):
            self.assertEqual(dualsub_from_srt.read(),
                             dualsub_expected.read())

    async def test_submerger_from_video(self) -> None:
        dualsub_from_video_path = TEST_OUTPUT_PATH / 'dualsub_from_video.srt'
        video_path = TEST_DATA_PATH / 'vid.mp4'

        return_code, _, _ = await execute_command(
            'submerge',
            '--video', video_path,
            '--language', 'eng', 'ger',
            '--output', dualsub_from_video_path,
        )
        self.assertEqual(return_code, 0)

        with (open(dualsub_from_video_path, encoding='utf-8')
                as dualsub_from_video,
                open(DUALSUB_EXPECTED_PATH, encoding='utf-8')
                as dualsub_expected):
            self.assertEqual(dualsub_from_video.read(),
                             dualsub_expected.read())

    async def test_submerger_global(self) -> None:
        dualsub_global_from_srt_path = TEST_GLOBAL_MERGING_PATH / 'ep1.srt'
        dualsub_global_from_video_path = TEST_GLOBAL_MERGING_PATH / 'ep2.srt'
        dualsub_expected_global_from_srt_path = TEST_GLOBAL_MERGING_PATH / 'ep1_expected.srt'
        dualsub_expected_global_from_video_path = TEST_GLOBAL_MERGING_PATH / 'ep2_expected.srt'

        expected_stdout = '''No eng subtitles for ep3
No ger subtitles for ep3

'''

        return_code, stdout, _ = await execute_command(
            'submerge',
            '--global',
            '--directory', TEST_GLOBAL_MERGING_PATH,
            '--language', 'eng', 'ger',
        )
        self.assertEqual(return_code, 0)
        self.assertEqual(stdout, expected_stdout)

        with (open(dualsub_global_from_srt_path, encoding='utf-8')
                as dualsub_global_from_srt,
                open(dualsub_expected_global_from_srt_path, encoding='utf-8')
                as dualsub_expected_from_srt,
                open(dualsub_global_from_video_path, encoding='utf-8')
                as dualsub_global_from_video,
                open(dualsub_expected_global_from_video_path, encoding='utf-8')
                as dualsub_expected_from_video):
            self.assertEqual(dualsub_global_from_srt.read(),
                             dualsub_expected_from_srt.read())
            self.assertEqual(dualsub_global_from_video.read(),
                             dualsub_expected_from_video.read())
